<?php

namespace App\Service;

use App\Exception as Exception;

class ConsumerService {

    private $util;
    private $factory;
    private $branchFactory;
    
    public function __construct (\App\Util\Util $util, 
                                 \App\Factory\ConsumerFactory $factory, 
                                 \App\Factory\BranchFactory $branchFactory) {

        $this->util          = $util;
        $this->factory       = $factory;
        $this->branchFactory = $branchFactory;
    }
    
    public function registerConsumer ($consumerDetails, $addressDetails, $generalParams) {
        $stdClient = $this->branchFactory->getStandardClient($generalParams["CDFILIAL"]);
        $consumerT = $this->branchFactory->getStandardConsumerType($generalParams["CDFILIAL"]);
        $consumer  = "SDE".$this->util->newCode("CONSUMIDOR".$stdClient["CDCLIENTE"], 18);
        
        $consumerDetails["CDCONSUMIDOR"] = $consumer;
        $mergeCandidates = array($stdClient, $consumerT);

        foreach ($mergeCandidates as $candidate) 
            $consumerDetails = array_merge($consumerDetails, $candidate);
        
        return $this->factory->registerConsumer(array_merge($consumerDetails, $addressDetails));
    }
    
    public function getIdConsumer($params) {
        $consumerIds     = $params;
        $arrayconsumerId = [];
        
        foreach ($consumerIds as $consumerId){
            if (!empty($consumerId)){
                array_push($arrayconsumerId,$consumerId);
            }
        }

        if (count($arrayconsumerId) != 1){
            $numParms = count($arrayconsumerId);
            throw new \Exception ("Foi enviado ". count($arrayconsumerId) . " parametros para consulta. É necessário enviar 1 parametro para consulta, podendo ser email, CPF/CNPJ ou cracha !");
        }

        if (!empty($consumerIds["email"])){
            $consumerEmail = $consumerIds["email"];
            $consumerIds   = $this->factory->getIdConsumerByEmail($consumerEmail);
        }
        
        if (!empty($consumerIds["cpf"])){
            if ($this->util->isValidCPF($consumerIds["cpf"])){
                $consumercpf = preg_replace( '/[^0-9]/is', '', $consumerIds["cpf"] );
                $consumerIds = $this->factory->getIdConsumerByCpf($consumercpf);
            }
            else{
                throw new \Exception ("CPF/CNPJ invalido");
            }
        }
        if (!empty($consumerIds["consumerRef"])){
            $consumerRef = $consumerIds["consumerRef"];
            $consumerIds = $this->factory->getIdConsumerByRef($consumerRef);
        }

        if (empty($consumerIds)){
            throw new \Exception ("Não foi encontrado nenhum registro com as informações enviadas.");
        }
        $arrayResp = [];
        foreach ($consumerIds as $consumer){
            $resp["clientId"]    = $consumer["CDCONSUMIDOR"];
            $resp["name"]        = $consumer["NMCONSUMIDOR"];
            $resp["email"]       = !empty($consumer["DSEMAILCONS"]) ? $consumer["DSEMAILCONS"] : "";
            $resp["phone"]       = !empty($consumer["NRTELECONS"]) ? $consumer["NRTELECONS"] :  "";
            $resp["cellPhone"]   = !empty($consumer["NRCELULARCONS"]) ? $consumer["NRCELULARCONS"] : "";
            $resp["cpf"]         = !empty($consumer["NRCPFRESPCON"]) ? $consumer["NRCPFRESPCON"] : "";
            $resp["consumerRef"] = !empty($consumer["CDIDCONSUMID"]) ? $consumer["CDIDCONSUMID"] : "";
            $resp["clientRef"]   = $consumer["CDCLIENTE"];

            array_push($arrayResp,$resp);
        }
        return $arrayResp;
    }
}
