<?php
namespace App\Service;

use App\Exception as Exception;

class PriceService {

    private $priceFactory;

    public function __construct(\App\Factory\PriceFactory $priceFactory) {
        
        $this->priceFactory = $priceFactory;
    }
    
    public function getPricingTable($CDCLIENTE, $CDFILIAL, $CDLOJA){
        try {
            $pricingTableGer = $this->priceFactory->generalPricingTable($CDFILIAL);
            $pricingTable = $this->priceFactory->clientBranchPricingTable($CDCLIENTE, $CDFILIAL);
            
            if(empty($pricingTable)) {
                $pricingTable = $this->priceFactory->clientPricingTable($CDCLIENTE);
                
                if (empty($pricingTable)) {
                    $pricingTable = $this->priceFactory->storePricingTable($CDFILIAL, $CDLOJA);
                    
                    if (empty($pricingTable)) {
                        $pricingTable = $this->priceFactory->paravendPricingTable($CDFILIAL);
                    }
                }
            }
            
            if(!empty($pricingTable) || !empty($pricingTableGer)) {
                if ($pricingTable['CDFILTABPREC'] === null) {
                    $pricingTable['CDFILTABPREC'] = $CDFILIAL;
                }
                
                if (!empty($pricingTableGer)) {
                    $pricingTable['CDTABEPRECGER'] = $pricingTableGer['CDTABEPRECGER'];
                    $pricingTable['DTINIVGPRECGER'] = $pricingTableGer['DTINIVGPRECGER'];
                }
            } else {
                throw new \UnexpectedValueException('Não há tabela de preços cadastrada.');
            }

            return $pricingTable;

        } catch (\UnexpectedValueException $e) {
            throw new \Exception($e->getMessage());
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    public function getGeneralPricingTable($CDFILIAL){
        
        $pricingTable = $this->priceFactory->getGeneralPricingTable($CDFILIAL);
            
        if(empty($pricingTable)) {
            throw new \Exception('Não há tabela de preços cadastrada.');
        }

        return $pricingTable['CDTABEPREC'];
    }
    
    public function checkEffective($CDFILIAL, $priceTable) {
        try {
            $salesTable = $this->priceFactory->getSalesTable($CDFILIAL, $priceTable);
            
            if (!empty($salesTable)) 
                return $salesTable['DTINIVGPREC'];
            else 
                throw new \UnexpectedValueException('Tabela de preços está inativa.');

        } catch (\UnexpectedValueException $e) {
            throw new \Exception($e->getMessage());
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    public function getGeneralPricingTableValidity($priceTable) {
        $salesTable = $this->priceFactory->getGeneralPricingTableValidity($priceTable);
        
        if (empty($salesTable)) 
            throw new \Exception('Validade da tabela de preços expirada ou inexistente.');

        return $salesTable['DTINIVGPREC'];
    }
    
    public function isHoliday($CDFILIAL, $currentDay) {
        try {
            $check = $this->priceFactory->holidayCheck($CDFILIAL, $currentDay);
            
            if (!empty($check)) 
                return true;
            
            return false;
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    public function getProductDiscountType($CDFILIAL, $CDTIPOCONS, $PRICINGTABLE, $effectiveDate = null) {
        return $this->priceFactory->getProductDiscountType($CDFILIAL, $CDTIPOCONS, $PRICINGTABLE, $effectiveDate);
    }
    
    public function getPricesOnHoliday($CDFILIAL, $CDTABEPREC, $DTINIVGPREC, $currentDay) {
        return $this->priceFactory->getPricesOnHoliday($CDFILIAL, $CDTABEPREC, $DTINIVGPREC, $currentDay);
    }
    
    public function getPricesByDay($CDFILIAL, $CDTABEPREC, $DTINIVGPREC, $currentDay, $CDTIPOCONSPD) {
        return $this->priceFactory->getPricesByDay($CDFILIAL, $CDTABEPREC, $DTINIVGPREC, $currentDay, $CDTIPOCONSPD);
    }
    
    public function getPrices($CDFILIAL, $CDFILTABPREC, $CDTABEPREC, $DTINIVGPREC) {
        try {

            $prices = $this->priceFactory->getPrices($CDFILIAL, $CDFILTABPREC, $CDTABEPREC, $DTINIVGPREC);
            if (!empty($prices['prices']) || !empty($prices['pricesGer'])) {
                $indexedPrices = array();
                $indexedPricesGer = array();
                $return = array();

                foreach ($prices['prices'] as $priceItem){
                    $index = $priceItem["CDPRODUTO"];
                    unset($priceItem["CDPRODUTO"]);
                    $indexedPrices[$index] = $priceItem;
                }
                
                foreach ($prices['pricesGer'] as $priceItemGer){
                    $index = $priceItemGer["CDPRODUTO"];
                    unset($priceItemGer["CDPRODUTO"]);
                    $indexedPricesGer[$index] = $priceItemGer;
                }
                
                $return = $indexedPrices;
                
                foreach ($indexedPricesGer as $key=>$item) {
                    if((!isset($return[$key])) && ($return[$key]['VRPRECITEM'] < 0.01)) {
                        $return[$key] = $item;
                        $return[$key]['PRECOGER'] = true;
                    }
                }
                return $return;
            } else {
                throw new \UnexpectedValueException("Não existem precos cadastrados.");
            }
        } catch (\UnexpectedValueException $e) {
            throw new \Exception($e->getMessage());
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    public function getGeneralPrices($CDTABEPREC, $DTINIVGPREC) {
        $prices = $this->priceFactory->getGeneralPrices($CDTABEPREC, $DTINIVGPREC);
        
        if (empty($prices)) {
            throw new \Exception("Não foi possível buscar precos associados a tabela de preco geral $CDTABEPREC.");
        }
        
        $indexedPrices = array();

        foreach ($prices as $priceItem){
            $index = $priceItem["CDPRODUTO"];
            $indexedPrices[$index] = $priceItem;
        }

        return $indexedPrices;
    }
    
    public function getDiffPrices($CDTABEPREC, $NRDIASEMANPR) {
        $prices = $this->priceFactory->getDiffPrices($CDTABEPREC, $NRDIASEMANPR);
        
        $indexedPrices = array();

        foreach ($prices as $priceItem){
            $index = $priceItem["CDPRODUTO"];
            unset($priceItem["CDPRODUTO"]);
            $indexedPrices[$index] = $priceItem;
        }

        return $indexedPrices;
    }
    
    public function checkPrice($CDFILIAL, $pricingTable, $productCode, $effectiveDate) {
        try {

            $itemPrice = $this->priceFactory->getItemPrice($CDFILIAL, $pricingTable, $effectiveDate, $productCode);

            if (empty($itemPrice)) {
                throw new \UnexpectedValueException('Preço não cadastrado.');
            } else if ($itemPrice['FULLPRICE']+$itemPrice['PRECOCLIE'] === 0) {
                throw new \UnexpectedValueException('Produto sem preço cadastrado.');
            } else {
                return $itemPrice;
            }

        } catch (\UnexpectedValueException $e) {
            throw new \Exception($e->getMessage());
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
}